from datetime import datetime
import logging
import os
import pathlib

import click

from plone_sync_s3.push import compress, upload
from plone_sync_s3.pull import download, extract


@click.group()
def cli():
    if os.getenv("DEBUG") is not None:
        logging.basicConfig(level="DEBUG")


@cli.command(help="Compress blobstorage and filestorage and upload to s3")
@click.argument("src", type=click.Path(exists=True))
@click.argument("s3_uri", type=str)
@click.option(
    "-n",
    "--filename",
    default=datetime.now().strftime("%Y%m%d-%H%M%S"),
)
def push(src: pathlib.Path, s3_uri: str, filename: str):
    tar_path = compress(src, filename)
    upload(tar_path, s3_uri)


@cli.command(help="Download tar file and extract")
@click.argument("s3_uri", type=str)
@click.argument("dist", type=click.Path())
def pull(s3_uri, dist: pathlib.Path):
    tarfile = download(s3_uri)
    extract(tarfile, dist)
    print(f"Finished. result file: {dist}")


if __name__ == "__main__":
    cli()



import logging
import pathlib
import tarfile
import tempfile
from urllib.parse import urlparse

import boto3
from botocore.exceptions import ClientError  # type: ignore
from mypy_boto3_s3.client import S3Client


def download(s3_uri: str) -> pathlib.Path:
    uri = urlparse(s3_uri)
    tar_path = pathlib.Path(uri.path.lstrip("/"))

    client: S3Client = boto3.client("s3")
    dist = pathlib.Path(tempfile.gettempdir()) / "plone-data" / tar_path.name
    try:
        with open(dist, mode="wb") as f:
            client.download_fileobj(Bucket=uri.netloc, Key=str(tar_path), Fileobj=f)
    except ClientError as e:
        logging.error(e)
    return dist


def extract(src: pathlib.Path, dist: pathlib.Path):
    with tarfile.open(src, "r") as f:
        f.extractall(dist)



import logging
import pathlib
import tarfile
from urllib.parse import urlparse

import boto3
from botocore.exceptions import ClientError  # type: ignore
from mypy_boto3_s3.client import S3Client


def compress(src: pathlib.Path, name: str) -> str:
    here = pathlib.Path()
    dist = here.resolve() / "data" / "backup" / f"{name}.tar"
    if not dist.exists():
        dist.parent.mkdir()
    with tarfile.open(dist, "w") as f:
        f.add(here / src / "blobstorage", arcname="blobstorage")
        f.add(here / src / "filestorage", arcname="filestorage")
    print(f"Compressed at {dist}")
    return str(dist)


def upload(tar_path: pathlib.Path, s3_uri: str):
    uri = urlparse(s3_uri)
    filename = pathlib.Path(tar_path).resolve().name
    s3_path = uri.path.lstrip("/")

    client: S3Client = boto3.client("s3")
    try:
        with open(tar_path, mode="rb") as f:
            client.upload_fileobj(f, Bucket=uri.netloc, Key=s3_path + filename)
            client.upload_fileobj(f, Bucket=uri.netloc, Key=s3_path + "latest.tar")
            print(f"Succeed upload to {s3_uri.rstrip('/')}/{filename}")
    except ClientError as e:
        logging.error(e)



import pathlib

from setuptools import setup, find_packages  # type: ignore

requires = [
    "click",
    "boto3",
    "boto3-stubs[s3]",
]
test_requires = [
    "pytest",
    "moto",
]
readme = open(pathlib.Path(__file__).parent.resolve() / "README.rst").read()

setup(
    name="mr.s3",
    version="1.0.0b3",
    description="Syncing scripts Plone data between local and AWS S3",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Framework :: Plone",
    ],
    author="Peacock",
    author_email="contact@peacock0803sz.com",
    url="https://github.com/peacock0803sz/mr.s3",
    keywords="Plone Python AWS",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    extras_require={"tests": test_requires},
    entry_points="""\
    [console_scripts]
    mr.s3 = mr_s3.router:cli
    """,
)
